//Wizard Include
namespace SpellType
{
	enum type
	{
		summoning,
		other
	};
}

shared class Spell
{
	string typeName;
	string name;
	string icon;
	u16 iconFrame;
	u8 type;
	s32 mana;

	s32 fullChargeTime;
	s32 readyTime;
	s32 cooldownTime;


	s32 ready_time;

	s32 cast_period;
	s32 cast_period_1;
	s32 cast_period_2;
	s32 full_cast_period;

	bool needs_full;

	Spell(string i_typeName, string i_name, u16 i_iconFrame, u8 i_type, s32 i_mana, s32 i_cast_period, s32 i_cooldownTime, bool fully_loaded = false)
	{
		typeName = i_typeName;
		name = i_name;
		iconFrame = i_iconFrame;
		type = i_type;
		mana = i_mana;
		cooldownTime = i_cooldownTime;

		cast_period = i_cast_period;
		cast_period_1 = cast_period/3;
		cast_period_2 = 2*cast_period/3;
		full_cast_period = cast_period*3;

		needs_full = fully_loaded;
	}
};

const string[] zombieTypes = {"zombie", "skeleton", "greg", "wraith"};

namespace WizardParams
{
	enum Aim {
		not_aiming = 0,
		charging,
		cast_1,
		cast_2,
		cast_3,
		extra_ready,
		}

	const ::f32 shoot_max_vel = 8.0f;

	const ::Spell[] spells = 
	{
		Spell("orb", "Orb", 5, SpellType::other, 2, 40, 120),
		Spell("teleport", "Teleportation", 7, SpellType::other, 20, 10, 480, true),
		Spell("counter_spell", "Counter Spell", 15, SpellType::other, 10, 15, 120, true),
		Spell("magic_missile", "Missiles of Magic", 16, SpellType::other, 20, 40, 120, true),
		Spell("frost_ball", "Ball of Frost", 12, SpellType::other, 15, 60, 120, true),
		Spell("heal", "Lesser Heal", 13, SpellType::other, 15, 20, 120, true), 
		Spell("firebomb", "Fire Bomb", 10, SpellType::other, 25, 70, 120, true),
		Spell("fire_sprite", "Fire Sprites", 11, SpellType::other, 15, 40, 120, true),		
		Spell("meteor_strike", "Meteor Strike", 8, SpellType::other, 40, 75, 120, true),
		Spell("revive", "Revive", 14, SpellType::other, 50, 80, 120, true),
		Spell("zombie", "Zombie", 1, SpellType::summoning, 20, 25, 30, true),
		Spell("skeleton_rain", "Skeleton Rain", 9, SpellType::other, 45, 90, 120),
		Spell("black_hole", "Black Hole", 17, SpellType::other, 40, 40, 30, true),
		Spell("greg", "Greg", 3, SpellType::summoning, 50, 50, 30, true),
		Spell("zombieknight", "Zombie Knight", 4, SpellType::summoning, 40, 100, 120, true)
	};
}

shared class WizardInfo
{
	s32 charge_time;
	u8 charge_state;
	s32 mana;
	s32 maxMana;
	u8 fletch_cooldown;
	u8 primarySpellID;
	u8 secondarySpellID;

	WizardInfo()
	{
		charge_time = 0;
		charge_state = 0;
		maxMana = 100;
		mana = maxMana;
		fletch_cooldown = 0;
		primarySpellID = 0;
		secondarySpellID = 1;
	}
}; 

void SetPrimarySpell( CBlob@ this, const u8 id )
{
	WizardInfo@ wizard;
	if (!this.get( "wizardInfo", @wizard ))
	{
		return;
	}
	wizard.primarySpellID = id;
}

void SetSecondarySpell( CBlob@ this, const u8 id )
{
	WizardInfo@ wizard;
	if (!this.get( "wizardInfo", @wizard ))
	{
		return;
	}
	wizard.secondarySpellID = id;
}

u8 getPrimarySpellID( CBlob@ this )
{
	WizardInfo@ wizard;
	if (!this.get( "wizardInfo", @wizard ))
	{
		return 0;
	}
	return wizard.primarySpellID;
}

u8 getSecondarySpellID( CBlob@ this )
{
	WizardInfo@ wizard;
	if (!this.get( "wizardInfo", @wizard ))
	{
		return 0;
	}
	return wizard.secondarySpellID;
}
