//archer HUD

#include "WizardCommon.as";
#include "WizardHUDStartPos.as";

const string iconsFilename = "SpellIcons.png";
const int slotsSize = 6;

void onInit( CSprite@ this )
{
	this.getCurrentScript().runFlags |= Script::tick_myplayer;
	this.getCurrentScript().removeIfTag = "dead";
	this.getBlob().set_u8("gui_HUD_slots_width", slotsSize);
}

void ManageCursors( CBlob@ this )
{
	// set cursor
	if (getHUD().hasButtons()) {
		getHUD().SetDefaultCursor();
	}
	else
	{
		// set cursor
		getHUD().SetCursorImage("WizardCursor.png", Vec2f(32,32));
		getHUD().SetCursorOffset( Vec2f(-32, -32) );
		// frame set in logic
	}
}

void DrawManaBar(CBlob@ this, Vec2f origin)
{
	WizardInfo@ wiz;
    if (!this.get( "wizardInfo", @wiz )) {
        return;
    }
    string manaFile = "GUI/ManaBar.png";
    int segmentWidth = 24;
    GUI::DrawIcon("GUI/jends.png", 0, Vec2f(8,16), origin+Vec2f(-8,0));
    int MANA = 0;
    s32 maxMana = wiz.maxMana;
    s32 mana = wiz.mana;
    for (f32 step = 0.0f; step < maxMana; step += 5.0f)
    {
        GUI::DrawIcon("GUI/ManaBack.png", 0, Vec2f(12,16), origin+Vec2f(segmentWidth*MANA,0));
        f32 thisMANA = mana - step;
        if (thisMANA > 0)
        {
            Vec2f manapos = origin+Vec2f(segmentWidth*MANA-1,0);
            if (thisMANA <= 1.0f) { GUI::DrawIcon(manaFile, 4, Vec2f(16,16), manapos); }
            else if (thisMANA <= 2.5f) { GUI::DrawIcon(manaFile, 3, Vec2f(16,16), manapos); }
            else if (thisMANA <= 4.5f) { GUI::DrawIcon(manaFile, 2, Vec2f(16,16), manapos); }
            else if (thisMANA > 4.5f) { GUI::DrawIcon(manaFile, 1, Vec2f(16,16), manapos); }
            else { GUI::DrawIcon(manaFile, 0, Vec2f(16,16), manapos); }
        }
        MANA++;
    }
	GUI::DrawText(""+mana+"/"+maxMana, origin+Vec2f(8,8), color_white );
    GUI::DrawIcon("GUI/jends.png", 1, Vec2f(8,16), origin+Vec2f(segmentWidth*MANA,0));
}

void DrawSpellBar(CBlob@ this)
{
	if (WizardParams::spells.length == 0) {
		return;
	}

    WizardInfo@ wizard;
    if (!this.get( "wizardInfo", @wizard )) {
        return;
    }

    Vec2f pos = Vec2f( 16.0f, getScreenHeight()-128.0f );

	const u8 primarySpell = wizard.primarySpellID;
	
	int spellsLength = WizardParams::spells.length;
	for (uint i = 0; i < spellsLength; i++)
	{
		Spell spell = WizardParams::spells[i];

		CBitStream params;
		params.write_u8(i);
		params.write_bool(false);
		
		f32 wizMana = wizard.mana;
		f32 spellMana = spell.mana;
		
		if ( i < 5 )
		{
			GUI::DrawFramedPane(pos + Vec2f(0,64) + Vec2f(32,0)*i, pos + Vec2f(32,96) + Vec2f(32,0)*i);
			GUI::DrawIcon("SpellIcons.png", spell.iconFrame, Vec2f(16,16), pos + Vec2f(0,64) + Vec2f(32,0)*i);
			GUI::DrawText(""+((i+1)%10), pos + Vec2f(8,-16) + Vec2f(32,0)*i, color_white );
			
			if ( wizard.primarySpellID == i )
				GUI::DrawRectangle(pos + Vec2f(0,64) + Vec2f(32,0)*i, pos + Vec2f(32,96) + Vec2f(32,0)*i, SColor(100, 0, 255, 0));
				
			if ( wizMana < spellMana )
				GUI::DrawRectangle(pos + Vec2f(0,64) + Vec2f(32,0)*i, pos + Vec2f(32*(1-(wizMana/spellMana)),96) + Vec2f(32,0)*i, SColor(100, 255, 0, 0));
		}
		else if ( i < 10 )
		{
			GUI::DrawFramedPane(pos + Vec2f(0,32) + Vec2f(32,0)*(i-5), pos + Vec2f(32,64) + Vec2f(32,0)*(i-5));
			GUI::DrawIcon("SpellIcons.png", spell.iconFrame, Vec2f(16,16), pos + Vec2f(0,32) + Vec2f(32,0)*(i-5));
			
			if ( wizard.primarySpellID == i )
				GUI::DrawRectangle(pos + Vec2f(0,32) + Vec2f(32,0)*(i-5), pos + Vec2f(32,64) + Vec2f(32,0)*(i-5), SColor(100, 0, 255, 0));
				
			if ( wizMana < spellMana )
				GUI::DrawRectangle(pos + Vec2f(0,32) + Vec2f(32,0)*(i-5), pos + Vec2f(32*(1-(wizMana/spellMana)),64) + Vec2f(32,0)*(i-5), SColor(100, 255, 0, 0));
		}
		else if ( i < 15 )
		{
			GUI::DrawFramedPane(pos + Vec2f(32,0)*(i-10), pos + Vec2f(32,32) + Vec2f(32,0)*(i-10));
			GUI::DrawIcon("SpellIcons.png", spell.iconFrame, Vec2f(16,16), pos + Vec2f(32,0)*(i-10));			
			
			if ( wizard.primarySpellID == i )
				GUI::DrawRectangle(pos + Vec2f(32,0)*(i-10), pos + Vec2f(32,32) + Vec2f(32,0)*(i-10), SColor(100, 0, 255, 0));
				
			if ( wizMana < spellMana )
				GUI::DrawRectangle(pos + Vec2f(32,0)*(i-10), pos + Vec2f(32*(1-(wizMana/spellMana)),32) + Vec2f(32,0)*(i-10), SColor(100, 255, 0, 0));
		}
		
		bool spellSelected = this.get_bool("spell selected");
		if ( spellSelected == false )
		{
			Vec2f arrowPosOffset = Vec2f(0,0);
		
			if ( (primarySpell == 0 ||  primarySpell == 5 ||  primarySpell == 10) )
				arrowPosOffset = Vec2f(0,0);
			else if ( (primarySpell == 1 ||  primarySpell == 6 ||  primarySpell == 11) )
				arrowPosOffset = Vec2f(32,0);
			else if ( (primarySpell == 2 ||  primarySpell == 7 ||  primarySpell == 12) )
				arrowPosOffset = Vec2f(64,0);
			else if ( (primarySpell == 3 ||  primarySpell == 8 ||  primarySpell == 13) )
				arrowPosOffset = Vec2f(96,0);
			else if ( (primarySpell == 4 ||  primarySpell == 9 ||  primarySpell == 14) )
				arrowPosOffset = Vec2f(128,0);
				
			GUI::DrawArrow2D( pos + Vec2f(14,-32) + arrowPosOffset, pos + Vec2f(14,-16) + arrowPosOffset, color_white);
		}
	}
	
	GUI::DrawPane(pos + Vec2f(32,96), pos + Vec2f(64,116) + Vec2f(32,0)*3, color_white);
	GUI::DrawText(WizardParams::spells[wizard.primarySpellID].name, pos + Vec2f(40,96), color_white );
	GUI::DrawText("Primary spell", pos + Vec2f(0,-48), color_white );
}

void onRender( CSprite@ this )
{
	if (g_videorecording)
		return;

	CBlob@ blob = this.getBlob();
	CPlayer@ player = blob.getPlayer();

	ManageCursors( blob );

	// draw inventory
	Vec2f tl = getActorHUDStartPosition(blob, slotsSize);
	DrawInventoryOnHUD( blob, tl, Vec2f(0,58));

	// draw coins
	const int coins = player !is null ? player.getCoins() : 0;
	DrawCoinsOnHUD( blob, coins, tl, slotsSize-2 );

	DrawManaBar(blob, Vec2f(52,64));
	DrawSpellBar(blob);

	// class weapon icon
	WizardInfo@ wiz;
    if (!blob.get( "wizardInfo", @wiz )) {
        return;
    }
	//GUI::DrawIcon( iconsFilename, wiz.primarySpellID, Vec2f(16,16), Vec2f(10,10));
	GUI::DrawIcon("GUI/jslot.png", 1, Vec2f(32,32), Vec2f(2,56));
	//GUI::DrawIcon( iconsFilename, wiz.secondarySpellID, Vec2f(16,16), Vec2f(10,66)); 
}
